/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: MyLeadSaver.java,v 1.10 2007/06/06 21:01:15 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.mylead.gui;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.gui.Cancelable;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.WaitDialog;
import edu.indiana.extreme.xbaya.mylead.MyLeadException;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class MyLeadSaver implements Cancelable {

    private static final MLogger logger = MLogger.getLogger();

    private XBayaEngine engine;

    private Thread loadThread;

    private boolean canceled;

    private WaitDialog savingDialog;

    /**
     * Constructs a MyLeadWorkflowLoader.
     * 
     * @param client
     */
    public MyLeadSaver(XBayaEngine client) {
        this.engine = client;

        this.savingDialog = new WaitDialog(this, "Saving the Workflow.",
                "Saving the Workflow. " + "Please wait for a moment.",
                this.engine);
    }

    /**
     * @see edu.indiana.extreme.xbaya.gui.Cancelable#cancel()
     */
    public void cancel() {
        this.canceled = true;
        this.loadThread.interrupt();
    }

    /**
     * Saves the workflow.
     * 
     * @param redeploy
     */
    public void save(final boolean redeploy) {
        this.canceled = false;

        final Workflow workflow = MyLeadSaver.this.engine.getWorkflow();
        this.loadThread = new Thread() {
            @Override
            public void run() {
                runInThread(redeploy, workflow);
            }
        };
        this.loadThread.start();

        // This has to be the last because it blocks when the dialog is modal.
        this.savingDialog.show();
    }

    /**
     * @param redeploy
     * @param workflow
     */
    private void runInThread(boolean redeploy, Workflow workflow) {
        try {
            this.engine.getGPELClient().createScriptAndDeploy(workflow,
                    redeploy);
            if (redeploy) {
                this.engine.getSubWorkflowUpdater().update(workflow);
            }
            if (this.canceled) {
                return;
            }
            this.engine.getMyLead().setProxy(
                    this.engine.getMyProxyClient().getProxy());
            this.engine.getMyLead().save(workflow, redeploy);
            this.savingDialog.hide();
        } catch (GraphException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.savingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.GRAPH_NOT_READY_ERROR, e);
            }
        } catch (GPELException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.savingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
        } catch (MyLeadException e) {
            if (MyLeadSaver.this.canceled) {
                logger.caught(e);
            } else {
                this.savingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MYLEAD_SAVE_TEMPLATE_ERROR, e);
            }
        } catch (ComponentException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.savingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            }
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.savingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MYLEAD_SAVE_TEMPLATE_ERROR, e);
            }
        } catch (Error e) {
            this.savingDialog.hide();
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
